import os
import shutil
import logging

logger = logging.getLogger(__name__)


class RoiSelector:

    VERTEBRAE_T4 = 'vertebrae_T4.nii.gz'
    VERTEBRAE_L2 = 'vertebrae_L2.nii.gz'
    VERTEBRAE_L3 = 'vertebrae_L3.nii.gz'
    VERTEBRAE_L4 = 'vertebrae_L4.nii.gz'

    def __init__(self):
        self.input_directory = None
        self.roi = None
        self.output_directory = None
        self.output_file = None

    def execute(self):
        if self.input_directory is None:
            logger.error('Input directory not specified')
            return None
        if self.roi is None:
            logger.error('ROI not specified')
            return None
        if self.output_directory is None:
            logger.error('Output directory not specified')
            return None
        os.makedirs(self.output_directory, exist_ok=True)
        shutil.copy(os.path.join(self.input_directory, RoiSelector.VERTEBRAE_L3, self.output_directory))
        self.output_file = os.path.join(self.output_directory, RoiSelector.VERTEBRAE_L3)
        return self.output_file



import os
import logging
import pydicom
import numpy as np

from barbell2.utils import calculate_area, calculate_mean_radiation_attenuation, get_pixels

logger = logging.getLogger(__name__)


class BodyCompositionCalculator:

    MUSCLE = 1
    VAT = 5
    SAT = 7

    def __init__(self):
        self.input_files = None                 # L3 images
        self.input_segmentation_files = None    # Segmentations calculated using MuscleFatSegmentator
        self.heights = None                     # (Optional) dictionary containing heights for each L3 image
        self.output_metrics = None              # Dictionary containing output metrics for each L3 image

    @staticmethod
    def load_dicom(f_path):
        p = pydicom.dcmread(f_path)
        pixel_spacing = p.PixelSpacing
        pixels = get_pixels(p, normalize=True)
        return pixels, pixel_spacing

    @staticmethod
    def load_segmentation(f_path):
        return np.load(f_path)

    def execute(self):
        if self.input_files is None:
            logger.error('Input files not specified')
            return None
        if self.input_segmentation_files is None:
            logger.error('Input segmentation files not specified')
            return None
        # Check that we're not dealing with probability maps
        if self.input_segmentation_files[0].endswith('.seg.prob.npy'):
            logger.error('Cannot handle *.seg.prob.npy files')
            return None
        # Check that for each input file we have a matching segmentation file
        file_pairs = []
        for input_file in self.input_files:
            input_file_name = os.path.split(input_file)[1]
            found = False
            for input_segmentation_file in self.input_segmentation_files:
                input_segmentation_file_name = os.path.split(input_segmentation_file)[1]
                if input_file_name + '.seg.npy' == input_segmentation_file_name:
                    file_pairs.append((input_file, input_segmentation_file))
                    found = True
                    break
            if not found:
                logger.warn(f'Input file {input_file_name} missing corresponding segmentation file')
        # Work with found file pairs
        self.output_metrics = {}
        for file_pair in file_pairs:
            image, pixel_spacing = self.load_dicom(file_pair[0])
            segmentations = self.load_segmentation(file_pair[1])
            self.output_metrics[file_pair[0]] = {
                'muscle_area': calculate_area(segmentations, BodyCompositionCalculator.MUSCLE, pixel_spacing),
                'vat_area': calculate_area(segmentations, BodyCompositionCalculator.VAT, pixel_spacing),
                'sat_area': calculate_area(segmentations, BodyCompositionCalculator.SAT, pixel_spacing),
                'muscle_ra': calculate_mean_radiation_attenuation(image, segmentations, BodyCompositionCalculator.MUSCLE),
                'vat_ra': calculate_mean_radiation_attenuation(image, segmentations, BodyCompositionCalculator.VAT),
                'sat_ra': calculate_mean_radiation_attenuation(image, segmentations, BodyCompositionCalculator.SAT),
            }
            logger.info(f'{file_pair[0]}:')
            logger.info(' - muscle_area: {}'.format(self.output_metrics[file_pair[0]]['muscle_area']))
            logger.info(' - vat_area: {}'.format(self.output_metrics[file_pair[0]]['muscle_area']))
            logger.info(' - sat_area: {}'.format(self.output_metrics[file_pair[0]]['muscle_area']))
            logger.info(' - muscle_ra: {}'.format(self.output_metrics[file_pair[0]]['muscle_area']))
            logger.info(' - vat_ra: {}'.format(self.output_metrics[file_pair[0]]['muscle_area']))
            logger.info(' - sat_ra: {}'.format(self.output_metrics[file_pair[0]]['muscle_area']))
        return self.output_metrics


if __name__ == '__main__':
    def main():
        calculator = BodyCompositionCalculator()
        calculator.input_files = ['/mnt/localscratch/cds/rbrecheisen/raw/pancreas-demo-1/1.dcm']
        calculator.input_segmentation_files = ['/tmp/barbell2/bodycomp/seg.py/1.dcm.seg.npy']
        output_metrics = calculator.execute()
        for k in output_metrics.keys():
            print(f'{k}: {output_metrics[k]}')
    main()



import json
import logging

from oauthlib.oauth2 import BackendApplicationClient
from requests_oauthlib import OAuth2Session

logger = logging.getLogger('__name__')

class CastorApiClient:

    base_url = 'https://data.castoredc.com'
    token_url = base_url + '/oauth/token'
    api_url = base_url + '/api'

    def __init__(self, client_id, client_secret, verbose=False):
        self.verbose = verbose
        if self.verbose:
            logger.info(f'__init__()')
        self.session = self.create_session(client_id, client_secret)
        self.studies = self.get_studies()

    def create_session(self, client_id, client_secret):
        """ Creates new session to communicate with Castor database REST API
        :param client_id
        :param client_secret
        """
        if self.verbose:
            logger.info(f'create_session('
                'client_id={client_id}, '
                'client_secret={client_secret}, '
                'token_url={self.token_url})'
            )
        client = BackendApplicationClient(client_id=client_id)
        client_session = OAuth2Session(client=client)
        client_session.fetch_token(
            token_url=self.token_url,
            client_id=client_id,
            client_secret=client_secret,
        )
        return client_session

    def get_studies(self):
        """ Returns list of study objects
        """
        uri = self.api_url + '/study'
        if self.verbose:
            print(f'get_studies() uri={uri}')
        response = self.session.get(uri)
        response_data = response.json()
        if self.verbose:
            print(f'get_studies() response_data={json.dumps(response_data, index=4)}')
        studies = []
        for study in response_data['_embedded']['study']:
            studies.append(study)
        return studies

    def get_study(self, name):
        """ Returns study object for given name
        :param name Study name
        """
        if self.verbose:
            print(f'get_study(name={name}')
        for study in self.studies:
            if study['name'] == name:
                return study
        return None

    def get_study_id(self, study):
        """ Returns study ID for given study object
        :param study Study object
        """
        if self.verbose:
            if 'study_id' not in study.keys():
                print(f'get_study_id(study={study}) missing key study_id')
        study_id = study['study_id']
        return study_id

    def get_records(self, study_id):
        """ Returns list of records
        :param study_id Study ID
        :param verbose
        """
        record_url = self.api_url + '/study/{}/record'.format(study_id)
        response = self.session.get(record_url)
        response_data = response.json()
        page_count = response_data['page_count']
        records = []
        for i in range(1, page_count + 1):
            record_page_url = self.api_url + '/study/{}/record?page={}'.format(study_id, i)
            response = self.session.get(record_page_url)
            response_data = response.json()
            for record in response_data['_embedded']['records']:
                if not record['id'].startswith('ARCHIVED'):
                    records.append(record)
                    if self.verbose:
                        print(record)
        return records

    @staticmethod
    def get_record_id(record):
        """ Returns record ID for given record object
        :param record Record object
        """
        record_id = record['id']
        return record_id

    def get_fields(self, study_id):
        """ Returns list of field objects defined for the given study
        :param study_id Study ID
        :param verbose
        """
        field_url = self.api_url + '/study/{}/field'.format(study_id)
        response = self.session.get(field_url)
        response_data = response.json()
        page_count = response_data['page_count']
        fields = []
        for i in range(1, page_count + 1):
            field_page_url = self.api_url + '/study/{}/field?page={}'.format(study_id, i)
            response = self.session.get(field_page_url)
            response_data = response.json()
            for field in response_data['_embedded']['fields']:
                fields.append(field)
                if self.verbose:
                    print(field)
        return fields

    @staticmethod
    def get_field(fields, name):
        """ Returns field object of given name
        :param fields List of field objects to search through
        :param name Field name
        """
        for field in fields:
            if field['field_variable_name'] == name:
                return field
        return None

    @staticmethod
    def get_field_id(field):
        """ Returns field ID for given field object
        :param field Field object
        """
        field_id = field['id']
        return field_id

    @staticmethod
    def get_field_type(field):
        """ Returns field type for given field object
        :param field Field object
        """
        field_type = field['field_type']
        return field_type

    def get_field_value(self, study_id, record_id, field_id):
        """ Returns value for given field, record and study
        :param study_id Study ID
        :param record_id Record ID
        :param field_id Field ID for which data is retrieved
        """
        field_data_url = self.api_url + '/study/{}/record/{}/study-data-point/{}'.format(study_id, record_id, field_id)
        response = self.session.get(field_data_url)
        response_data = response.json()
        # Check for 'value' key because if field is empty, there will be no 'value' key
        if 'value' in response_data.keys():
            return response_data['value']
        return None



import pandas as pd


class CastorReportBuilder:

    HTML = 0

    def __init__(self, df, output_format=HTML):
        self.df = df
        self.output_format = output_format

    def execute(self):
        pass


if __name__ == '__main__':
    def main():
        builder = CastorReportBuilder()
        builder.execute()
    main()



import os
import subprocess
import logging

logger = logging.getLogger(__name__)


class TotalSegmentator:

    def __init__(self):
        try:
            subprocess.call(['TotalSegmentator'])
        except FileNotFoundError:
            logger.error(
                'TotalSegmentator is not installed!\n'
                'Please install it using pip install pytorch totalsegmentator'
            )
        self.input_file = None
        self.output_directory = None
        self.fast = False
        self.statistics = False
        self.radiomics = False
        self.cmd = None

    def execute(self):
        if self.input_file is None:
            logger.error('Input NIFTI file not specified')
            return None
        if self.output_directory is None:
            logger.error('Output directory not specified')
            return None
        fast = ''
        if self.fast:
            fast = '--fast'
        statistics = ''
        if self.statistics:
            statistics = '--statistics'
        radiomics = ''
        if self.radiomics:
            radiomics = '--radiomics'
        self.cmd = 'TotalSegmentator {} {} {} -i {} -o {}'.format(
            statistics, 
            radiomics,
            fast,
            self.input_file,
            self.output_directory,
        )
        logger.info(f'Running command: {self.cmd}')
        os.system(self.cmd)
        return self.output_directory


# class TotalSegmentator:

#     def __init__(self, nifti_path, output_dir):
#         self.nifti_path = nifti_path
#         self.output_dir = output_dir
#         self.fast = False
#         self.statistics = True
#         self.radiomics = False
#         # Check installation of Total Segmentator 
#         try:
#             subprocess.call(['TotalSegmentator'])
#         except FileNotFoundError:
#             logger.error(
#                 'TotalSegmentator is not installed!\n'
#                 'Please install it using pip install pytorch totalsegmentator'
#             )

#     def execute(self):
#         fast = ''
#         if self.fast:
#             fast = '--fast'
#         statistics = ''
#         if self.statistics:
#             statistics = '--statistics'
#         radiomics = ''
#         if self.radiomics:
#             radiomics = '--radiomics'
#         # Build command
#         cmd = 'TotalSegmentator {} {} {} -i {} -o {}'.format(
#             statistics, 
#             radiomics,
#             fast,
#             self.nifti_path,
#             self.output_dir,
#         )
#         logger.info(f'Running command: {cmd}')
#         os.system(cmd)


if __name__ == '__main__':
    def main():
        ts = TotalSegmentator('', '')
    main()
